{% load i18n activity_tags %}
<h2>{% include 'actstream/action.html' %}</h2>
<p>{% blocktrans with date=action.timestamp|date time=action.timestamp|time %}On {{ date }} at {{ time }}{% endblocktrans %}</p>
