<html>
    <head>{% block extra_head %}{% endblock %}</head>
    <body>
        <h1>{% block title %}Django Activity Stream{% endblock %}</h1>
        {% block content %}{% endblock %}
    </body>
</html>
